@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004, 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2004-2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -31,27 +31,22 @@
 #include <stdarg.h>
 #include <assert.h>
 
-#include "prelude-log.h"
-
 #define PRELUDE_ERROR_SOURCE_DEFAULT PRELUDE_ERROR_SOURCE_IDMEF_CRITERIA
-#include "prelude-error.h"
-#include "prelude-inttypes.h"
-
-#include "idmef.h"
+#include "prelude.h"
 #include "idmef-criteria.h"
 
 
 struct idmef_criterion {
-	idmef_path_t *path;
-	idmef_criterion_value_t *value;
-	idmef_criterion_operator_t operator;
+        idmef_path_t *path;
+        idmef_criterion_value_t *value;
+        idmef_criterion_operator_t operator;
 };
 
 
 struct idmef_criteria {
-	prelude_list_t list;
-        
-	idmef_criterion_t *criterion;
+        prelude_list_t list;
+
+        idmef_criterion_t *criterion;
         struct idmef_criteria *or;
         struct idmef_criteria *and;
 };
@@ -74,7 +69,10 @@ const char *idmef_criterion_operator_to_string(idmef_criterion_operator_t op)
                 const char *name;
         } tbl[] = {
                 { IDMEF_CRITERION_OPERATOR_EQUAL,     "="            },
+                { IDMEF_CRITERION_OPERATOR_EQUAL_NOCASE, "=*"        },
+
                 { IDMEF_CRITERION_OPERATOR_NOT_EQUAL, "!="           },
+                { IDMEF_CRITERION_OPERATOR_NOT_EQUAL_NOCASE, "!=*"   },
 
                 { IDMEF_CRITERION_OPERATOR_LESSER, "<"               },
                 { IDMEF_CRITERION_OPERATOR_GREATER, ">"              },
@@ -85,17 +83,17 @@ const char *idmef_criterion_operator_to_string(idmef_criterion_operator_t op)
                 { IDMEF_CRITERION_OPERATOR_REGEX_NOCASE, "~*"        },
                 { IDMEF_CRITERION_OPERATOR_NOT_REGEX, "!~"           },
                 { IDMEF_CRITERION_OPERATOR_NOT_REGEX_NOCASE, "!~*"   },
-                                
+
                 { IDMEF_CRITERION_OPERATOR_SUBSTR, "<>"              },
                 { IDMEF_CRITERION_OPERATOR_SUBSTR_NOCASE, "<>*"      },
                 { IDMEF_CRITERION_OPERATOR_NOT_SUBSTR, "!<>"         },
                 { IDMEF_CRITERION_OPERATOR_NOT_SUBSTR_NOCASE, "!<>*" },
-                
+
                 { IDMEF_CRITERION_OPERATOR_NOT_NULL, "!"             },
                 { IDMEF_CRITERION_OPERATOR_NULL, ""                  },
         };
 
-        for ( i = 0; tbl[i].operator != 0; i++ ) 
+        for ( i = 0; tbl[i].operator != 0; i++ )
                 if ( op == tbl[i].operator )
                         return tbl[i].name;
 
@@ -120,18 +118,18 @@ const char *idmef_criterion_operator_to_string(idmef_criterion_operator_t op)
 int idmef_criterion_new(idmef_criterion_t **criterion, idmef_path_t *path,
                         idmef_criterion_value_t *value, idmef_criterion_operator_t op)
 {
-        if ( ! value && ! (op & IDMEF_CRITERION_OPERATOR_NULL) )
-                return -1;
-        
-	*criterion = calloc(1, sizeof(**criterion));
-	if ( ! *criterion )
-		return prelude_error_from_errno(errno);
+        prelude_return_val_if_fail(path != NULL, -1);
+        prelude_return_val_if_fail(! (value == NULL && ! (op & IDMEF_CRITERION_OPERATOR_NULL)), -1);
 
-	(*criterion)->path = path;
-	(*criterion)->value = value;
-	(*criterion)->operator = op;
+        *criterion = calloc(1, sizeof(**criterion));
+        if ( ! *criterion )
+                return prelude_error_from_errno(errno);
 
-	return 0;
+        (*criterion)->path = path;
+        (*criterion)->value = value;
+        (*criterion)->operator = op;
+
+        return 0;
 }
 
 
@@ -144,12 +142,14 @@ int idmef_criterion_new(idmef_criterion_t **criterion, idmef_path_t *path,
  */
 void idmef_criterion_destroy(idmef_criterion_t *criterion)
 {
-	idmef_path_destroy(criterion->path);
+        prelude_return_if_fail(criterion);
 
-	if ( criterion->value ) /* can be NULL if operator is is_null or is_not_null */
-		idmef_criterion_value_destroy(criterion->value);
+        idmef_path_destroy(criterion->path);
 
-	free(criterion);
+        if ( criterion->value ) /* can be NULL if operator is is_null or is_not_null */
+                idmef_criterion_value_destroy(criterion->value);
+
+        free(criterion);
 }
 
 
@@ -163,32 +163,34 @@ void idmef_criterion_destroy(idmef_criterion_t *criterion)
  *
  * Returns: 0 on success, a negative value if an error occured.
  */
-int idmef_criterion_clone(idmef_criterion_t *criterion, idmef_criterion_t **dst)
+int idmef_criterion_clone(const idmef_criterion_t *criterion, idmef_criterion_t **dst)
 {
         int ret;
-	idmef_path_t *path;
-	idmef_criterion_value_t *value = NULL;
+        idmef_path_t *path;
+        idmef_criterion_value_t *value = NULL;
 
-	ret = idmef_path_clone(criterion->path, &path);
-	if ( ret < 0 )
-		return ret;
+        prelude_return_val_if_fail(criterion, -1);
 
-	if ( criterion->value ) {
-		ret = idmef_criterion_value_clone(criterion->value, &value);
-		if ( ret < 0 ) {
-			idmef_path_destroy(path);
-			return ret;
-		}
-	} 
+        ret = idmef_path_clone(criterion->path, &path);
+        if ( ret < 0 )
+                return ret;
 
-	ret = idmef_criterion_new(dst, path, value, criterion->operator);
-	if ( ret < 0 ) {
-		idmef_path_destroy(path);
-		idmef_criterion_value_destroy(value);
-		return ret;
-	}
+        if ( criterion->value ) {
+                ret = idmef_criterion_value_clone(criterion->value, &value);
+                if ( ret < 0 ) {
+                        idmef_path_destroy(path);
+                        return ret;
+                }
+        }
 
-	return 0;
+        ret = idmef_criterion_new(dst, path, value, criterion->operator);
+        if ( ret < 0 ) {
+                idmef_path_destroy(path);
+                idmef_criterion_value_destroy(value);
+                return ret;
+        }
+
+        return 0;
 }
 
 
@@ -211,6 +213,8 @@ int idmef_criterion_print(const idmef_criterion_t *criterion, prelude_io_t *fd)
         int ret;
         prelude_string_t *out;
 
+        prelude_return_val_if_fail(criterion && fd, -1);
+
         ret = prelude_string_new(&out);
         if ( ret < 0 )
                 return ret;
@@ -246,11 +250,14 @@ int idmef_criterion_to_string(const idmef_criterion_t *criterion, prelude_string
 {
         const char *name, *operator;
 
+        prelude_return_val_if_fail(criterion && out, -1);
+
         operator = idmef_criterion_operator_to_string(criterion->operator);
-        assert(operator);
+        if ( ! operator )
+                return -1;
 
         name = idmef_path_get_name(criterion->path, -1);
-        
+
         if ( ! criterion->value )
                 return prelude_string_sprintf(out, "%s %s", operator, name);
 
@@ -269,9 +276,10 @@ int idmef_criterion_to_string(const idmef_criterion_t *criterion, prelude_string
  *
  * Returns: the #idmef_path_t object associated with @criterion.
  */
-idmef_path_t *idmef_criterion_get_path(idmef_criterion_t *criterion)
+idmef_path_t *idmef_criterion_get_path(const idmef_criterion_t *criterion)
 {
-	return criterion->path;
+        prelude_return_val_if_fail(criterion, NULL);
+        return criterion->path;
 }
 
 
@@ -286,9 +294,10 @@ idmef_path_t *idmef_criterion_get_path(idmef_criterion_t *criterion)
  *
  * Returns: the #idmef_criterion_value_t object associated with @criterion.
  */
-idmef_criterion_value_t *idmef_criterion_get_value(idmef_criterion_t *criterion)
+idmef_criterion_value_t *idmef_criterion_get_value(const idmef_criterion_t *criterion)
 {
-	return criterion ? criterion->value : NULL;
+        prelude_return_val_if_fail(criterion, NULL);
+        return criterion->value;
 }
 
 
@@ -302,8 +311,9 @@ idmef_criterion_value_t *idmef_criterion_get_value(idmef_criterion_t *criterion)
  *
  * Returns: the #idmef_criterion_operator_t associated with @criterion.
  */
-idmef_criterion_operator_t idmef_criterion_get_operator(idmef_criterion_t *criterion)
+idmef_criterion_operator_t idmef_criterion_get_operator(const idmef_criterion_t *criterion)
 {
+        prelude_return_val_if_fail(criterion, -1);
         return criterion->operator;
 }
 
@@ -320,29 +330,35 @@ idmef_criterion_operator_t idmef_criterion_get_operator(idmef_criterion_t *crite
  *
  * Returns: 1 for a match, 0 for no match, or a negative value if an error occured.
  */
-int idmef_criterion_match(idmef_criterion_t *criterion, idmef_message_t *message)
+int idmef_criterion_match(const idmef_criterion_t *criterion, idmef_message_t *message)
 {
         int ret;
-	idmef_value_t *value;
-	
-	ret = idmef_path_get(criterion->path, message, &value);
+        idmef_value_t *value;
+
+        prelude_return_val_if_fail(criterion && message, -1);
+
+        ret = idmef_path_get(criterion->path, message, &value);
         if ( ret < 0 )
                 return ret;
-        
-        if ( ret == 0 )
-		return (criterion->operator == IDMEF_CRITERION_OPERATOR_NULL) ? 1 : 0;
-        
+
+        if ( ret == 0 ) {
+                if ( criterion->value && criterion->operator & IDMEF_CRITERION_OPERATOR_NOT )
+                        return 1;
+
+                return (criterion->operator == IDMEF_CRITERION_OPERATOR_NULL) ? 1 : 0;
+        }
+
         if ( ! criterion->value ) {
                 idmef_value_destroy(value);
                 return (criterion->operator == (IDMEF_CRITERION_OPERATOR_NULL|IDMEF_CRITERION_OPERATOR_NOT)) ? 1 : 0;
         }
-        
+
         ret = idmef_criterion_value_match(criterion->value, value, criterion->operator);
         idmef_value_destroy(value);
 
         if ( ret < 0 )
                 return ret;
-        
+
         return (ret > 0) ? 1 : 0;
 }
 
@@ -359,13 +375,13 @@ int idmef_criterion_match(idmef_criterion_t *criterion, idmef_message_t *message
 int idmef_criteria_new(idmef_criteria_t **criteria)
 {
         *criteria = calloc(1, sizeof(**criteria));
-	if ( ! *criteria )
-		return prelude_error_from_errno(errno);
+        if ( ! *criteria )
+                return prelude_error_from_errno(errno);
 
         (*criteria)->or = NULL;
         (*criteria)->and = NULL;
 
-	return 0;
+        return 0;
 }
 
 
@@ -378,16 +394,18 @@ int idmef_criteria_new(idmef_criteria_t **criteria)
  */
 void idmef_criteria_destroy(idmef_criteria_t *criteria)
 {
-	if ( criteria->criterion )
-		idmef_criterion_destroy(criteria->criterion);
+        prelude_return_if_fail(criteria);
 
-	if ( criteria->or )
-		idmef_criteria_destroy(criteria->or);
+        if ( criteria->criterion )
+                idmef_criterion_destroy(criteria->criterion);
 
-	if ( criteria->and )
-		idmef_criteria_destroy(criteria->and);
-        
-	free(criteria);
+        if ( criteria->or )
+                idmef_criteria_destroy(criteria->or);
+
+        if ( criteria->and )
+                idmef_criteria_destroy(criteria->and);
+
+        free(criteria);
 }
 
 
@@ -405,59 +423,65 @@ int idmef_criteria_clone(idmef_criteria_t *src, idmef_criteria_t **dst)
 {
         int ret;
         idmef_criteria_t *new;
-        
+
+        prelude_return_val_if_fail(src, -1);
+
         new = *dst = malloc(sizeof(*new));
         if ( ! new )
                 return prelude_error_from_errno(errno);
 
         memcpy(new, src, sizeof(*new));
 
-	if ( src->or ) {
-		ret = idmef_criteria_clone(src->or, &new->or);
-		if ( ret < 0 ) {
-			idmef_criteria_destroy(new);
-			return ret;
-		}
-	}
+        if ( src->or ) {
+                ret = idmef_criteria_clone(src->or, &new->or);
+                if ( ret < 0 ) {
+                        idmef_criteria_destroy(new);
+                        return ret;
+                }
+        }
+
+        if ( src->and ) {
+                ret = idmef_criteria_clone(src->and, &new->and);
+                if ( ret < 0 ) {
+                        idmef_criteria_destroy(new);
+                        return ret;
+                }
+        }
 
-	if ( src->and ) {
-		ret = idmef_criteria_clone(src->and, &new->and);
-		if ( ret < 0 ) {
-			idmef_criteria_destroy(new);
-			return ret;
-		}
-	}
-        
         ret = idmef_criterion_clone(src->criterion, &new->criterion);
         if ( ret < 0 ) {
                 idmef_criteria_destroy(new);
                 return ret;
         }
-        
+
         return 0;
 }
 
 
 
-idmef_criteria_t *idmef_criteria_get_or(idmef_criteria_t *criteria)
+idmef_criteria_t *idmef_criteria_get_or(const idmef_criteria_t *criteria)
 {
+        prelude_return_val_if_fail(criteria, NULL);
         return criteria->or;
 }
 
 
 
-idmef_criteria_t *idmef_criteria_get_and(idmef_criteria_t *criteria)
+idmef_criteria_t *idmef_criteria_get_and(const idmef_criteria_t *criteria)
 {
+        prelude_return_val_if_fail(criteria, NULL);
         return criteria->and;
 }
 
 
 
-int idmef_criteria_print(idmef_criteria_t *criteria, prelude_io_t *fd)
+int idmef_criteria_print(const idmef_criteria_t *criteria, prelude_io_t *fd)
 {
         int ret;
         prelude_string_t *out;
 
+        prelude_return_val_if_fail(criteria && fd, -1);
+
         ret = prelude_string_new(&out);
         if ( ret < 0 )
                 return ret;
@@ -474,42 +498,43 @@ int idmef_criteria_print(idmef_criteria_t *criteria, prelude_io_t *fd)
 
 
 
-int idmef_criteria_to_string(idmef_criteria_t *criteria, prelude_string_t *out)
+int idmef_criteria_to_string(const idmef_criteria_t *criteria, prelude_string_t *out)
 {
-	if ( ! criteria )
-		return -1;
+        prelude_return_val_if_fail(criteria && out, -1);
 
-	if ( criteria->or )
+        if ( criteria->or )
                 prelude_string_sprintf(out, "((");
 
         idmef_criterion_to_string(criteria->criterion, out);
-        
+
         if ( criteria->and ) {
                 prelude_string_sprintf(out, " && ");
-		idmef_criteria_to_string(criteria->and, out);
+                idmef_criteria_to_string(criteria->and, out);
         }
-        
+
         if ( criteria->or ) {
                 prelude_string_sprintf(out, ") || (");
-		idmef_criteria_to_string(criteria->or, out);
+                idmef_criteria_to_string(criteria->or, out);
                 prelude_string_sprintf(out, "))");
         }
-        
-	return 0;
+
+        return 0;
 }
 
 
 
-prelude_bool_t idmef_criteria_is_criterion(idmef_criteria_t *criteria)
+prelude_bool_t idmef_criteria_is_criterion(const idmef_criteria_t *criteria)
 {
-	return (criteria->criterion != NULL) ? TRUE : FALSE;
+        prelude_return_val_if_fail(criteria, FALSE);
+        return (criteria->criterion != NULL) ? TRUE : FALSE;
 }
 
 
 
-idmef_criterion_t *idmef_criteria_get_criterion(idmef_criteria_t *criteria)
+idmef_criterion_t *idmef_criteria_get_criterion(const idmef_criteria_t *criteria)
 {
-	return criteria->criterion;
+        prelude_return_val_if_fail(criteria, NULL);
+        return criteria->criterion;
 }
 
 
@@ -517,12 +542,14 @@ idmef_criterion_t *idmef_criteria_get_criterion(idmef_criteria_t *criteria)
 void idmef_criteria_or_criteria(idmef_criteria_t *criteria, idmef_criteria_t *criteria2)
 {
         idmef_criteria_t *last = NULL;
-        
+
+        prelude_return_if_fail(criteria && criteria2);
+
         while ( criteria ) {
                 last = criteria;
                 criteria = criteria->or;
         }
-        
+
         last->or = criteria2;
 }
 
@@ -532,23 +559,25 @@ int idmef_criteria_and_criteria(idmef_criteria_t *criteria, idmef_criteria_t *cr
 {
         int ret;
         idmef_criteria_t *new, *last = NULL;
-        
+
+        prelude_return_val_if_fail(criteria && criteria2, -1);
+
         while ( criteria ) {
                 last = criteria;
-                
+
                 if ( criteria->or ) {
                         ret = idmef_criteria_clone(criteria2, &new);
                         if ( ret < 0 )
                                 return ret;
-                        
+
                         ret = idmef_criteria_and_criteria(criteria->or, new);
                         if ( ret < 0 )
                                 return ret;
                 }
-                
+
                 criteria = criteria->and;
         }
-        
+
         last->and = criteria2;
 
         return 0;
@@ -559,6 +588,7 @@ int idmef_criteria_and_criteria(idmef_criteria_t *criteria, idmef_criteria_t *cr
 
 void idmef_criteria_set_criterion(idmef_criteria_t *criteria, idmef_criterion_t *criterion)
 {
+        prelude_return_if_fail(criteria && criterion);
         criteria->criterion = criterion;
 }
 
@@ -574,22 +604,24 @@ void idmef_criteria_set_criterion(idmef_criteria_t *criteria, idmef_criterion_t
  *
  * Returns: 1 if criteria match, 0 if it did not, a negative value if an error occured.
  */
-int idmef_criteria_match(idmef_criteria_t *criteria, idmef_message_t *message)
+int idmef_criteria_match(const idmef_criteria_t *criteria, idmef_message_t *message)
 {
         int ret;
         idmef_criteria_t *next;
-        
+
+        prelude_return_val_if_fail(criteria && message, -1);
+
         ret = idmef_criterion_match(criteria->criterion, message);
         if ( ret < 0 )
                 return ret;
-        
+
         if ( ret == 0 )
                 next = criteria->or;
         else
                 next = criteria->and;
-        
+
         if ( ! next )
                 return ret;
-                
+
         return idmef_criteria_match(next, message);
 }
