@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004, 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2004-2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -36,6 +36,7 @@
 #include "prelude-error.h"
 #include "prelude-client-profile.h"
 #include "tls-auth.h"
+#include "common.h"
 
 #ifdef WIN32
 # define geteuid(x) (0)
@@ -73,28 +74,37 @@ struct prelude_client_profile {
 
 
 
-static int get_profile_analyzerid(prelude_client_profile_t *cp) 
+static int get_profile_analyzerid(prelude_client_profile_t *cp)
 {
         int ret;
         FILE *fd;
         char *ptr, filename[256], buf[256];
 
+        prelude_client_profile_get_profile_dirname(cp, filename, sizeof(filename));
+        if ( access(filename, R_OK|X_OK) < 0 ) {
+                if ( errno == ENOENT )
+                        return prelude_error_verbose(PRELUDE_ERROR_PROFILE, "profile '%s' does not exist", cp->name);
+
+                else if ( errno == EACCES )
+                        return prelude_error_verbose(PRELUDE_ERROR_PROFILE, "could not open profile '%s': insufficient permission", cp->name);
+        }
+
         prelude_client_profile_get_analyzerid_filename(cp, filename, sizeof(filename));
-        
+
         fd = fopen(filename, "r");
         if ( ! fd )
                 return prelude_error_verbose(PRELUDE_ERROR_PROFILE, "could not open '%s' for reading", filename);
 
         ptr = fgets(buf, sizeof(buf), fd);
         fclose(fd);
-        
+
         if ( ! ptr )
                 return prelude_error_verbose(PRELUDE_ERROR_PROFILE, "could not read analyzerID from '%s'", filename);
-        
+
         ret = sscanf(buf, "%" PRELUDE_PRIu64, &cp->analyzerid);
         if ( ret != 1 )
                 return prelude_error_verbose(PRELUDE_ERROR_PROFILE, "'%s' is not a valid analyzerID", buf);
-        
+
         return 0;
 }
 
@@ -108,8 +118,9 @@ static int get_profile_analyzerid(prelude_client_profile_t *cp)
  *
  * Writes the filename used to store @cp unique and permanent analyzer ident.
  */
-void prelude_client_profile_get_analyzerid_filename(const prelude_client_profile_t *cp, char *buf, size_t size) 
+void prelude_client_profile_get_analyzerid_filename(const prelude_client_profile_t *cp, char *buf, size_t size)
 {
+        prelude_return_if_fail(cp && buf);
         snprintf(buf, size, PRELUDE_PROFILE_DIR "/%s/analyzerid", cp->name);
 }
 
@@ -123,8 +134,9 @@ void prelude_client_profile_get_analyzerid_filename(const prelude_client_profile
  *
  * Writes the filename used to store @cp configuration template.
  */
-void prelude_client_profile_get_config_filename(const prelude_client_profile_t *cp, char *buf, size_t size) 
+void prelude_client_profile_get_config_filename(const prelude_client_profile_t *cp, char *buf, size_t size)
 {
+        prelude_return_if_fail(cp && buf);
         snprintf(buf, size, PRELUDE_PROFILE_DIR "/%s/config", cp->name);
 }
 
@@ -138,8 +150,9 @@ void prelude_client_profile_get_config_filename(const prelude_client_profile_t *
  *
  * Writes the filename used to store @cp private key.
  */
-void prelude_client_profile_get_tls_key_filename(const prelude_client_profile_t *cp, char *buf, size_t size) 
+void prelude_client_profile_get_tls_key_filename(const prelude_client_profile_t *cp, char *buf, size_t size)
 {
+        prelude_return_if_fail(cp && buf);
         snprintf(buf, size, PRELUDE_PROFILE_DIR "/%s/key", cp->name);
 }
 
@@ -154,8 +167,9 @@ void prelude_client_profile_get_tls_key_filename(const prelude_client_profile_t
  * Writes the filename used to store @cp related CA certificate.
  * This only apply to @cp receiving connection from analyzer (server).
  */
-void prelude_client_profile_get_tls_server_ca_cert_filename(const prelude_client_profile_t *cp, char *buf, size_t size) 
+void prelude_client_profile_get_tls_server_ca_cert_filename(const prelude_client_profile_t *cp, char *buf, size_t size)
 {
+        prelude_return_if_fail(cp && buf);
         snprintf(buf, size, PRELUDE_PROFILE_DIR "/%s/server.ca", cp->name);
 }
 
@@ -170,8 +184,9 @@ void prelude_client_profile_get_tls_server_ca_cert_filename(const prelude_client
  * Writes the filename used to store certificate for @cp server.
  * This only apply to @cp receiving connection from analyzer (server).
  */
-void prelude_client_profile_get_tls_server_keycert_filename(const prelude_client_profile_t *cp, char *buf, size_t size) 
+void prelude_client_profile_get_tls_server_keycert_filename(const prelude_client_profile_t *cp, char *buf, size_t size)
 {
+        prelude_return_if_fail(cp && buf);
         snprintf(buf, size, PRELUDE_PROFILE_DIR "/%s/server.keycrt", cp->name);
 }
 
@@ -186,8 +201,9 @@ void prelude_client_profile_get_tls_server_keycert_filename(const prelude_client
  * Writes the filename used to store CRL for @cp server.
  * This only apply to @cp receiving connection from analyzer (server).
  */
-void prelude_client_profile_get_tls_server_crl_filename(const prelude_client_profile_t *cp, char *buf, size_t size) 
+void prelude_client_profile_get_tls_server_crl_filename(const prelude_client_profile_t *cp, char *buf, size_t size)
 {
+        prelude_return_if_fail(cp && buf);
         snprintf(buf, size, PRELUDE_PROFILE_DIR "/%s/server.crl", cp->name);
 }
 
@@ -202,8 +218,9 @@ void prelude_client_profile_get_tls_server_crl_filename(const prelude_client_pro
  * Writes the filename used to store peers public certificates that @cp trust.
  * This only apply to client connecting to a peer.
  */
-void prelude_client_profile_get_tls_client_trusted_cert_filename(const prelude_client_profile_t *cp, char *buf, size_t size) 
+void prelude_client_profile_get_tls_client_trusted_cert_filename(const prelude_client_profile_t *cp, char *buf, size_t size)
 {
+        prelude_return_if_fail(cp && buf);
         snprintf(buf, size, PRELUDE_PROFILE_DIR "/%s/client.trusted", cp->name);
 }
 
@@ -219,8 +236,9 @@ void prelude_client_profile_get_tls_client_trusted_cert_filename(const prelude_c
  * Writes the filename used to store public certificate for @cp private key.
  * This only apply to client connecting to a peer.
  */
-void prelude_client_profile_get_tls_client_keycert_filename(const prelude_client_profile_t *cp, char *buf, size_t size) 
+void prelude_client_profile_get_tls_client_keycert_filename(const prelude_client_profile_t *cp, char *buf, size_t size)
 {
+        prelude_return_if_fail(cp && buf);
         snprintf(buf, size, PRELUDE_PROFILE_DIR "/%s/client.keycrt", cp->name);
 }
 
@@ -235,8 +253,9 @@ void prelude_client_profile_get_tls_client_keycert_filename(const prelude_client
  * Writes the directory name where message sent by @cp will be stored,
  * in case writing the message to the peer fail.
  */
-void prelude_client_profile_get_backup_dirname(const prelude_client_profile_t *cp, char *buf, size_t size) 
+void prelude_client_profile_get_backup_dirname(const prelude_client_profile_t *cp, char *buf, size_t size)
 {
+        prelude_return_if_fail(cp && buf);
         snprintf(buf, size, PRELUDE_SPOOL_DIR "/%s", cp->name);
 }
 
@@ -249,8 +268,9 @@ void prelude_client_profile_get_backup_dirname(const prelude_client_profile_t *c
  *
  * Writes the directory name where the profile for @client is stored.
  */
-void prelude_client_profile_get_profile_dirname(const prelude_client_profile_t *cp, char *buf, size_t size) 
+void prelude_client_profile_get_profile_dirname(const prelude_client_profile_t *cp, char *buf, size_t size)
 {
+        prelude_return_if_fail(cp && buf);
         snprintf(buf, size, PRELUDE_PROFILE_DIR "/%s", cp->name);
 }
 
@@ -263,7 +283,7 @@ int _prelude_client_profile_new(prelude_client_profile_t **ret)
 
         (*ret)->uid = geteuid();
         (*ret)->gid = getegid();
-        
+
         return 0;
 }
 
@@ -272,7 +292,9 @@ int _prelude_client_profile_new(prelude_client_profile_t **ret)
 int _prelude_client_profile_init(prelude_client_profile_t *cp)
 {
         int ret;
-        
+
+        prelude_return_val_if_fail(cp, -1);
+
         ret = get_profile_analyzerid(cp);
         if ( ret < 0 )
                 return ret;
@@ -297,10 +319,12 @@ int prelude_client_profile_new(prelude_client_profile_t **ret, const char *name)
         int retval;
         prelude_client_profile_t *cp;
 
+        prelude_return_val_if_fail(name, -1);
+
         retval = _prelude_client_profile_new(&cp);
         if ( retval < 0 )
                 return retval;
-        
+
         cp->name = strdup(name);
         if ( ! cp->name ) {
                 free(cp);
@@ -310,9 +334,9 @@ int prelude_client_profile_new(prelude_client_profile_t **ret, const char *name)
         retval = _prelude_client_profile_init(cp);
         if ( retval < 0 )
                 return retval;
-        
+
         *ret = cp;
-        
+
         return 0;
 }
 
@@ -325,10 +349,12 @@ int prelude_client_profile_new(prelude_client_profile_t **ret, const char *name)
  * Destroys @cp.
  */
 void prelude_client_profile_destroy(prelude_client_profile_t *cp)
-{        
+{
+        prelude_return_if_fail(cp);
+
         if ( cp->credentials )
                 gnutls_certificate_free_credentials(cp->credentials);
-        
+
         if ( cp->name )
                 free(cp->name);
 
@@ -347,6 +373,7 @@ void prelude_client_profile_destroy(prelude_client_profile_t *cp)
  */
 uid_t prelude_client_profile_get_uid(const prelude_client_profile_t *cp)
 {
+        prelude_return_val_if_fail(cp, 0);
         return cp->uid;
 }
 
@@ -361,6 +388,7 @@ uid_t prelude_client_profile_get_uid(const prelude_client_profile_t *cp)
  */
 void prelude_client_profile_set_uid(prelude_client_profile_t *cp, uid_t uid)
 {
+        prelude_return_if_fail(cp);
         cp->uid = uid;
 }
 
@@ -376,6 +404,7 @@ void prelude_client_profile_set_uid(prelude_client_profile_t *cp, uid_t uid)
  */
 gid_t prelude_client_profile_get_gid(const prelude_client_profile_t *cp)
 {
+        prelude_return_val_if_fail(cp, 0);
         return cp->gid;
 }
 
@@ -390,6 +419,7 @@ gid_t prelude_client_profile_get_gid(const prelude_client_profile_t *cp)
  */
 void prelude_client_profile_set_gid(prelude_client_profile_t *cp, gid_t gid)
 {
+        prelude_return_if_fail(cp);
         cp->gid = gid;
 }
 
@@ -404,6 +434,7 @@ void prelude_client_profile_set_gid(prelude_client_profile_t *cp, gid_t gid)
  */
 void prelude_client_profile_set_analyzerid(prelude_client_profile_t *cp, uint64_t analyzerid)
 {
+        prelude_return_if_fail(cp);
         cp->analyzerid = analyzerid;
 }
 
@@ -419,6 +450,7 @@ void prelude_client_profile_set_analyzerid(prelude_client_profile_t *cp, uint64_
  */
 uint64_t prelude_client_profile_get_analyzerid(const prelude_client_profile_t *cp)
 {
+        prelude_return_val_if_fail(cp, 0);
         return cp->analyzerid;
 }
 
@@ -434,6 +466,7 @@ uint64_t prelude_client_profile_get_analyzerid(const prelude_client_profile_t *c
  */
 const char *prelude_client_profile_get_name(const prelude_client_profile_t *cp)
 {
+        prelude_return_val_if_fail(cp, NULL);
         return cp->name;
 }
 
@@ -442,16 +475,18 @@ const char *prelude_client_profile_get_name(const prelude_client_profile_t *cp)
  * prelude_client_profile_set_name:
  * @cp: Pointer to a #prelude_client_profile_t object.
  * @name: Name to associate the profile with.
- * 
+ *
  * Sets the prelude client profile name.
- * 
+ *
  * Returns: 0 on success or a negative value if an error occured.
  */
 int prelude_client_profile_set_name(prelude_client_profile_t *cp, const char *name)
 {
+        prelude_return_val_if_fail(cp && name, -1);
+
         if ( cp->name )
                 free(cp->name);
-        
+
         cp->name = strdup(name);
         if ( ! cp->name )
                 return prelude_error_from_errno(errno);
@@ -465,25 +500,27 @@ int prelude_client_profile_set_name(prelude_client_profile_t *cp, const char *na
  * prelude_client_profile_get_crendentials:
  * @cp: Pointer to a #prelude_client_profile_t object.
  * @credentials: The GNU TLS certificate credentials retrieved.
- * 
+ *
  * Gets the prelude client profile credentials
- * 
+ *
  * Returns: 0 on success or a negative value if an error occured.
  */
 int prelude_client_profile_get_credentials(prelude_client_profile_t *cp, void **credentials)
 {
         int ret;
-        
+
+        prelude_return_val_if_fail(cp, -1);
+
         if ( cp->credentials ) {
                 *credentials = cp->credentials;
                 return 0;
         }
-        
-        ret = tls_auth_init(cp, &cp->credentials);        
+
+        ret = tls_auth_init(cp, &cp->credentials);
         if ( ret < 0 )
                 return ret;
 
         *credentials = cp->credentials;
-        
+
         return 0;
 }
