@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2001,2002,2003,2004,2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2001-2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -24,9 +24,11 @@
 #include "libmissing.h"
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
 #include <errno.h>
+#include <time.h>
 
 #ifndef WIN32
 # include <syslog.h>
@@ -34,37 +36,41 @@
 # include <windows.h>
 #endif
 
+#include "common.h"
 #include "prelude-log.h"
 #include "prelude-error.h"
 #include "prelude-inttypes.h"
 
 
-static void do_log_print(prelude_log_t level, const char *str);
+#if ! defined (PRELUDE_VA_COPY)
+
+# if defined (__GNUC__) && defined (__PPC__) && (defined (_CALL_SYSV) || defined (_WIN32))
+#  define PRELUDE_VA_COPY(ap1, ap2)     (*(ap1) = *(ap2))
+
+# elif defined (PRELUDE_VA_COPY_AS_ARRAY)
+#  define PRELUDE_VA_COPY(ap1, ap2)     memmove ((ap1), (ap2), sizeof(va_list))
+
+# else /* va_list is a pointer */
+#  define PRELUDE_VA_COPY(ap1, ap2)     ((ap1) = (ap2))
+
+# endif
+#endif
+
+
+static int do_log_print(prelude_log_t level, const char *str);
 
 
 static int debug_level = 0;
+static int raise_abort_level = 0;
+prelude_bool_t raise_abort_set = FALSE;
 static int log_level = PRELUDE_LOG_INFO;
 
-
 static char *global_prefix = NULL;
+
 static FILE *debug_logfile = NULL;
 static prelude_log_flags_t log_flags = 0;
-static void (*global_log_cb)(prelude_log_t level, const char *str) = do_log_print;
-
-
-
-static inline FILE *get_out_fd(prelude_log_t level)
-{        
-        return (level < PRELUDE_LOG_INFO) ? stderr : stdout;
-}
-
-
-
-static void do_log_print(prelude_log_t level, const char *str)
-{
-        FILE *out = get_out_fd(level);
-        fprintf(out, "%s", str);
-}
+static int (*global_log_cb)(prelude_log_t level, const char *str) = do_log_print;
+static void (*external_log_cb)(prelude_log_t level, const char *str) = NULL;
 
 
 #ifdef WIN32
@@ -88,18 +94,63 @@ static void syslog_win32(int priority, const char *log)
 #endif
 
 
-static void do_log_syslog(prelude_log_t level, const char *str)
+static int do_log_syslog(prelude_log_t level, const char *str)
 {
+        int slevel = LOG_INFO;
+
+        if ( level == PRELUDE_LOG_CRIT )
+                slevel = LOG_CRIT;
+
+        else if ( level == PRELUDE_LOG_ERR )
+                slevel = LOG_ERR;
+
+        else if ( level == PRELUDE_LOG_WARN )
+                slevel = LOG_WARNING;
+
+        else if ( level == PRELUDE_LOG_INFO )
+                slevel = LOG_INFO;
+
+        else if ( level >= PRELUDE_LOG_DEBUG )
+                slevel = LOG_DEBUG;
+
         while (*str == '\n' ) str++;
 
 #ifndef WIN32
-        syslog(level, "%s", str);
+        syslog(slevel, "%s", str);
 #else
-        syslog_win32(level, str);
+        syslog_win32(slevel, str);
 #endif
+
+        return 0;
 }
 
 
+static inline FILE *get_out_fd(prelude_log_t level)
+{
+        return (level < PRELUDE_LOG_INFO) ? stderr : stdout;
+}
+
+
+
+static int do_log_print(prelude_log_t level, const char *str)
+{
+        int ret;
+        FILE *out = get_out_fd(level);
+
+        ret = fprintf(out, "%s", str);
+        if ( ret < 0 && errno == EBADF )
+                return -1;
+
+        return 0;
+}
+
+
+static int do_log_external(prelude_log_t level, const char *str)
+{
+        external_log_cb(level, str);
+        return 0;
+}
+
 
 static inline prelude_bool_t need_to_log(prelude_log_t level, prelude_log_t cur)
 {
@@ -107,50 +158,103 @@ static inline prelude_bool_t need_to_log(prelude_log_t level, prelude_log_t cur)
 }
 
 
+static const char *level_to_string(prelude_log_t level)
+{
+        if ( level >= PRELUDE_LOG_DEBUG )
+                return "DEBUG";
+
+        else if ( level == PRELUDE_LOG_ERR )
+                return "ERROR";
+
+        else if ( level == PRELUDE_LOG_INFO )
+                return "INFO";
+
+        else if ( level == PRELUDE_LOG_WARN )
+                return "WARNING";
+
+        else if ( level == PRELUDE_LOG_CRIT )
+                return "CRITICAL";
+
+        return "";
+}
+
+
+static ssize_t get_header(prelude_log_t level, char *buf, size_t size)
+{
+        int ret;
+        struct tm *t;
+        size_t len = 0;
+        time_t now = time(NULL);
+
+        buf[0] = 0;
+
+        if ( ! (log_flags & PRELUDE_LOG_FLAGS_SYSLOG) ) {
+                t = localtime(&now);
+                if ( t )
+                        len = strftime(buf, size, "%d %b %H:%M:%S ", t);
+
+                ret = snprintf(buf + len, size - len, "(process:%d) %s: ", getpid(), level_to_string(level));
+                if ( ret < 0 || ret >= (size - len) )
+                        return -1;
+        } else {
+                ret = snprintf(buf + len, size - len, "%s: ", level_to_string(level));
+                if ( ret < 0 || ret >= (size - len) )
+                        return -1;
+        }
+
+        return (size_t) len + ret;
+}
+
 
 static void do_log_v(prelude_log_t level, const char *file,
                      const char *function, int line, const char *fmt, va_list ap)
 {
-        ssize_t len;
+        int ret;
+        va_list bkp;
         char buf[1024];
-        
-        if ( level >= PRELUDE_LOG_DEBUG || level == PRELUDE_LOG_ERR ) {
+        ssize_t len = 0;
 
-                len = snprintf(buf, sizeof(buf), "%s%s:%s:%d: ", (global_prefix) ?
-                               global_prefix : "", file, function, line);
-                if ( len < 0 || len >= sizeof(buf) )
+        if ( global_log_cb == do_log_print || global_log_cb == do_log_syslog ) {
+                len = get_header(level, buf, sizeof(buf));
+                if ( len < 0 )
                         return;
-                
-                vsnprintf(buf + len, sizeof(buf) - len, fmt, ap);
         }
 
-        else {
-                len = snprintf(buf, sizeof(buf), "%s", (global_prefix) ? global_prefix : "");
-                if ( len < 0 || len >= sizeof(buf) )
-                        return;
-                
-                vsnprintf(buf + len, sizeof(buf) - len, fmt, ap);
+        PRELUDE_VA_COPY(bkp, ap);
+
+        ret = vsnprintf(buf + len, sizeof(buf) - len, fmt, ap);
+        if ( ret < 0 || ret >= (sizeof(buf) - len) )
+                goto out;
+
+        if ( level <= PRELUDE_LOG_ERR || level >= PRELUDE_LOG_DEBUG ) {
+                for ( len += ret; buf[len - 1] == '\n' ; len-- );
+                snprintf(buf + len, sizeof(buf) - len, " (%s:%d %s)\n", file, line, function);
         }
 
-        if ( need_to_log(level, log_level) )
-                global_log_cb(level, buf);
-        
-        else if ( need_to_log(level, debug_level) ) {
+        if ( need_to_log(level, log_level) || need_to_log(level, debug_level) ) {
+                ret = global_log_cb(level, buf);
+                if ( ret < 0 && global_log_cb == do_log_print ) {
+                        prelude_log_set_flags(PRELUDE_LOG_FLAGS_SYSLOG);
+                        do_log_v(level, file, function, line, fmt, bkp);
+                        goto out;
+                }
+
                 if ( debug_logfile )
                         fprintf(debug_logfile, "%s", buf);
-                else
-                        global_log_cb(level, buf);
         }
+
+out:
+        va_end(bkp);
 }
 
 
 
 void _prelude_log_v(prelude_log_t level, const char *file,
-                    const char *function, int line, const char *fmt, va_list ap) 
+                    const char *function, int line, const char *fmt, va_list ap)
 {
         if ( ! need_to_log(level, log_level) && ! need_to_log(level, debug_level) )
                 return;
-        
+
         do_log_v(level, file, function, line, fmt, ap);
 }
 
@@ -169,16 +273,19 @@ void _prelude_log_v(prelude_log_t level, const char *file,
  * Use the #log macro defined in prelude-log.h
  */
 void _prelude_log(prelude_log_t level, const char *file,
-                  const char *function, int line, const char *fmt, ...) 
+                  const char *function, int line, const char *fmt, ...)
 {
         va_list ap;
-        
+
         if ( ! need_to_log(level, log_level) && ! need_to_log(level, debug_level) )
                 return;
-        
+
         va_start(ap, fmt);
-        do_log_v(level, file, function, line, fmt, ap);        
+        do_log_v(level, file, function, line, fmt, ap);
         va_end(ap);
+
+        if ( raise_abort_set && level == raise_abort_level )
+                abort();
 }
 
 
@@ -187,7 +294,7 @@ void _prelude_log(prelude_log_t level, const char *file,
  * prelude_log_set_flags:
  * @flags:
  */
-void prelude_log_set_flags(prelude_log_flags_t flags) 
+void prelude_log_set_flags(prelude_log_flags_t flags)
 {
         if ( flags & PRELUDE_LOG_FLAGS_QUIET )
                 log_level = PRELUDE_LOG_WARN;
@@ -196,7 +303,7 @@ void prelude_log_set_flags(prelude_log_flags_t flags)
                 global_log_cb = do_log_syslog;
         else
                 global_log_cb = do_log_print;
-        
+
         log_flags = flags;
 }
 
@@ -228,14 +335,14 @@ void prelude_log_set_debug_level(int level)
  * Tell the Prelude standard logger to add @prefix before logging
  * a line.
  */
-void prelude_log_set_prefix(char *prefix) 
+void prelude_log_set_prefix(char *prefix)
 {
         global_prefix = prefix;
 }
 
 
 
-char *prelude_log_get_prefix(void) 
+char *prelude_log_get_prefix(void)
 {
         return global_prefix;
 }
@@ -251,7 +358,8 @@ char *prelude_log_get_prefix(void)
  */
 void prelude_log_set_callback(void log_cb(prelude_log_t level, const char *str))
 {
-        global_log_cb = log_cb;
+        external_log_cb = log_cb;
+        global_log_cb = do_log_external;
 }
 
 
@@ -263,16 +371,59 @@ int prelude_log_set_logfile(const char *filename)
                 debug_logfile = NULL;
         }
 
-        else {        
+        else {
                 debug_logfile = fopen(filename, "a");
                 if ( ! debug_logfile )
                         return prelude_error_from_errno(errno);
         }
-        
+
         return 0;
 }
 
 
+void _prelude_log_set_abort_level(prelude_log_t level)
+{
+        raise_abort_set = TRUE;
+        raise_abort_level = level;
+}
+
+
+int _prelude_log_set_abort_level_from_string(const char *level)
+{
+        int i;
+        char *eptr;
+        long lvalue;
+        struct {
+                const char *prefix;
+                prelude_log_t level;
+        } tbl[] = {
+                { "CRIT",  PRELUDE_LOG_CRIT  },
+                { "ERR",   PRELUDE_LOG_ERR   },
+                { "WARN",  PRELUDE_LOG_WARN  },
+                { "INFO",  PRELUDE_LOG_INFO  },
+                { "DEBUG", PRELUDE_LOG_DEBUG }
+        };
+
+        prelude_return_val_if_fail(level != NULL, -1);
+
+        for ( i = 0; i < sizeof(tbl) / sizeof(*tbl); i++ ) {
+                if ( strncasecmp(tbl[i].prefix, level, strlen(tbl[i].prefix)) == 0 ) {
+                        _prelude_log_set_abort_level(tbl[i].level);
+                        return 0;
+                }
+        }
+
+        lvalue = strtol(level, &eptr, 10);
+        if ( eptr != (level + strlen(level)) || lvalue == LONG_MIN || lvalue == LONG_MAX ) {
+                prelude_log(PRELUDE_LOG_WARN, "Invalid abort level specified: '%s'.\n", level);
+                return -1;
+        }
+
+        _prelude_log_set_abort_level(lvalue);
+
+        return 0;
+}
+
 
 #ifndef HAVE_VARIADIC_MACROS
 
@@ -284,7 +435,7 @@ void prelude_log(prelude_log_t level, const char *fmt, ...)
         _prelude_log_v(level, "", "", 0, fmt, ap);
         va_end(ap);
 }
-         
+
 
 void prelude_log_debug(prelude_log_t level, const char *fmt, ...)
 {
