@@ -22,6 +22,7 @@
 *****/
 
 #include "config.h"
+#include "libmissing.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -254,8 +255,8 @@ inline static int slice_message_header(prelude_msg_t *msg, unsigned char *hdrbuf
                 return prelude_error(PRELUDE_ERROR_INVAL_LENGTH);
 
         if ( (msg->hdr.datalen + tmp) >= max_message_size )
-                return prelude_error_verbose(PRELUDE_ERROR_INVAL_LENGTH, "maximum message size exceeded: %lu > %lu",
-                                             (msg->hdr.datalen + tmp), max_message_size);
+                return prelude_error_verbose(PRELUDE_ERROR_INVAL_LENGTH, "maximum message size exceeded: %" PRELUDE_PRIu64 " > %" PRELUDE_PRIu64,
+                                             (uint64_t) (msg->hdr.datalen + tmp), (uint64_t) max_message_size);
         
         msg->hdr.datalen += prelude_extract_uint32(hdrbuf + 4);
         
